# meds/bots/cli.py
#
#

""" console line interface bot. """

__copyright__ = "Copyright 2015, Bart Thate"

## IMPORTS

from meds.defines import GREEN, ENDC
from meds.kernel import kernel
from meds.object import Object
from meds.event import Event
from meds.bots import BotBase
from meds.cfg import cfg

import time
import sys

## CLI

class CLI(BotBase):

    """ console line interface bot. """

    cc = "none"
    default = ""

    def out(self, txt):
        """ output to sys.stdout. """
        sys.stdout.write(str(txt))
        sys.stdout.write("\n")

    def say(self, channel, txt):
        """ say something. """
        self.out(txt)

    def prompt(self):
        """ wait for user input. """
        self.wait()
        if cfg.kernel.colors: return "%s %s<%s " % (time.ctime(time.time()).split()[-2], GREEN, ENDC)
        else: return "%s < " % (time.ctime(time.time()).split()[-2])

    def start(self, *args, **kwargs):
        """ start the bot. """
        kernel.fleet.append(self)
        if not cfg.kernel.shell:
            thrs = self.cmnd(" ".join(cfg.kernel.runargs))
            if thrs: self.collect(thrs)
            self.ready()
            return
        return BotBase.start(self, *args, **kwargs)

    def event(self, txt=""):
        """ return event. """
        self.ready()
        event = Event()
        event.txt = input(self.prompt())
        event._bot = self
        event.origin = "root@shell"
        return event

    def dispatch(self, *args, **kwargs):
        """ move task to dispatcher. """
        return kernel.dispatch(*args, **kwargs)

    def cmnd(self, txt):
        """ run a cmnd on the commands dispatcher. """
        event = Event()
        event.txt = txt
        event.origin = "user@bot"
        event._bot = self
        return kernel.dispatch(event)
