# event.py
#
#

""" object passed on to commands. """

__copyright__ = "Copyright 2015, Bart Thate"

## IMPORTS

from meds.utils.misc import days, names
from meds.object import Object

import time

## OBJECT

class Event(Object):

    """ object passed on to cmnds. """

    ## OUTPUT

    def reply(self, txt):
        """ send a reply to orginiating channel. """
        if "channel" in self: self.say(self.channel, txt)
        elif "origin" in self: self.say(self.origin, txt)
        else: self.say("", txt)

    def say(self, channel, txt):
        """ say text on a channel. """ 
        if "outer" in self: self.outer.write(str(txt) + "\n") ; self.outer.flush() ; return
        if "_bot" not in self: raise BotNotSet()
        self._bot.say(channel, txt)

    def ok(self, txt=""):
        """ reply with 'ok' and some descriptive text if provided. """
        self.reply("ok %s" % txt)

    def show(self, obj, sep="\n"):
        """ return the data of the Object in key=value pairs. """
        self.reply("".join(["%s=%s%s" % (a, obj[a], sep) for a in sorted(names(obj))]))

    def display(self, obj, keys, txt=""):
        """ return a string constructed based on a list of keys. Include days ago as well. """
        for key in keys:
            try: 
                val = str(getattr(obj, str(key), ""))
                if val: txt += " " + val
            except: pass
        txt = txt.rstrip() 
        txt += " - %s" % days(obj)
        self.reply(txt)

