# meds/services/rss.py
#
#

""" rss module. """

__copyright__ = "Copyright 2015, Bart Thate"

## IMPORTS

from meds.utils.url import get_url, strip_html, need_redirect, get_feed
from meds.utils.time import short_date
from meds.utils.other import matching
from meds.utils.lock import locked
from meds.utils.misc import slice
from meds.clock import Repeater
from meds.object import Object
from meds.kernel import kernel
from meds.errors import error
from meds.store import Store
from meds.cfg import cfg

import logging
import time

## DEFINES

seen = Object()
seen.list = []
seen.results = "rss"
seen.prefix = "results"

## DISPLAY

def display(*args, **kwargs):
    entry = args[0]
    result = ""
    for key in cfg.rss.keys_list:
        if key not in entry: continue
        if key == "summary": data = strip_html(entry[key])
        else: data = entry[key]
        if entry.get(key, None): result += "%s - " % data
    return result[:-3]

## FETCH

def fetch(*args, **kwargs):
    global seen
    obj = args[0]
    counter = 0
    for o in get_feed(obj.rss):
        if o.link in seen.list: continue
        seen.list.append(o.link)
        s = slice(o, cfg.rss.keys_list)
        s.services = "rss"
        s.prefix = "feeds"
        s.short = short_date(time.ctime())
        if not matching(cfg.rss.nosave, o.link): s.save()
        kernel.announce(display(s))
        counter += 1
    return counter
    
def rss_fetch(*args, **kwargs):
    result = []
    thrs = []
    store = Store()
    for fn in store.all("rss"):
        obj = Object().load(fn)
        if "deleted" in obj and obj.deleted: continue
        if "rss" not in obj: continue
        if "http" not in obj.rss: continue
        if cfg.rss.ignore and cfg.rss.ignore in obj.rss: continue
        thr = kernel.start(fetch, obj)
        if thr: thrs.append(thr)
    for thr in thrs: result.append(thr.join(15.0))
    logging.info("# RSS/fetch %s" % sum([x for x in result if x]))
    seen.sync()
    return result

kernel.register("rss.fetch", rss_fetch)

## INIT

def init(*args, **kwargs):
    global seen
    store = Store()
    rss = store.last("results", "rss")
    if rss: seen = rss
    rss = Repeater(600, rss_fetch)
    kernel.start(rss.start)

