# test.py
#
#

""" test commands and classes. """

__copyright__ = "Copyright 2015, Bart Thate"

## IMPORTS

from meds.utils.misc import names
from meds.utils.name import typed
from meds.object import Object
from meds.kernel import kernel
from meds.errors import EOWNER
from meds.bots import BotBase
from meds.event import Event
from meds.cfg import cfg

## IMPORTS

import unittest
import logging
import time
import sys
import os

if not os.getcwd() in sys.path: sys.path.append(os.getcwd())

## TestBot

class TestBot(BotBase):

    """ test bot. """

    testing = True
    cc = "!"
    default = ""
    channels = ["#test", ]

    def out(self, txt):
        if "verbose" in self and self.verbose: print(txt)

    def say(self, channel, txt): self.out(txt)

    def event(self, txt=""):
        event = Event()
        event.txt = txt
        event._bot = self
        event.origin = "test@bot"
        return event

## Test 

class Test(unittest.TestCase): pass

## DEFINES

exclude = ["fetch", "test"]
outtxt = u"Đíť ìš éèñ ëņċøďıńğŧęŝţ· .. にほんごがはなせません .. ₀0⁰₁1¹₂2²₃3³₄4⁴₅5⁵₆6⁶₇7⁷₈8⁸₉9⁹ .. ▁▂▃▄▅▆▇▉▇▆▅▄▃▂▁ .. .. uǝʌoqǝʇsɹǝpuo pɐdı ǝɾ ʇpnoɥ ǝɾ"

## COMMANDS

def forcedreconnect(event):
    """ no arguments - do a forced reconnect. """
    for bot in kernel.fleet:
        try: bot._sock.shutdown(2)
        except: event.reply("%s bot doesn't have a _sock attribute" % typed(bot))

kernel.register('forcedreconnect', forcedreconnect)

def forcedexception(event):
    """ no arguments - raise a exception. """
    if event.origin != cfg.kernel.owner: event.reply("EOWNER %s" % event.origin) ; return 
    event.reply("raising exception.")
    raise Exception('test exception')

kernel.register('forcedexception', forcedexception)

def wrongxml(event):
    """ no arguments - try sending borked xml. """
    event.reply('sending bork xml')
    for bot in kernel.fleet: bot.out('<message asdfadf/>')

kernel.register('wrongxml', wrongxml)

def test_xmlescape(event):
    """ no arguments - try sending borked xml. """
    event.reply('sending xml')
    event.reply('<message asdfadf/>')

kernel.register('xmlescape', test_xmlescape)

def unicode(event):
    """ no arguments - send unicode test down the output paths. """
    event.reply(outtxt)

kernel.register('unicode', unicode)

def deadline(event):
    """ no arguments - slee for 40 seconds in the kernelloop. """
    try: nrseconds = int(event.rest)
    except: event.reply("need number of seconds to sleep.") ; return
    event.reply('starting %s sec sleep' % nrseconds)
    time.sleep(nrseconds)

kernel.register('deadline', deadline)

def html(event):
    """ arguments: [<txt>] - test the html=True option to event.reply(). """
    event.reply('<span style="font-family: fixed; font-size: 10pt"><b>YOOOO BROEDERS</b></span>')

kernel.register('html', html)

def runcmnds(event):
    if cfg.kernel.owner not in  event.origin: event.reply("EOWNER %s" % event.origin) ; return
    bot = TestBot()
    bot.workdir = "test.data"
    try: nrloops = int(event.rest)
    except: nrloops = 1
    for x in range(nrloops):
        for cmnd in sorted(list(names(kernel.cmnds))):
            e = bot.event("!" + cmnd)
            if "runcmnds" in e.txt: continue
            logging.warn("< %s" % e.txt)
            kernel.start(kernel.dispatch, e)

kernel.register("runcmnds", runcmnds)
