# store.py
#
#

""" misc. functions. """

from meds.utils.time import fn_time, rtime
from meds.utils.trace import get_location

import logging
import hashlib
import types
import time
import json
import os

## DIGEST

def signature(obj):
    """ create a signature of this Object. """
    return str(hashlib.sha1(bytes(str(obj), "utf-8")).hexdigest())

## JSON

def dumps(obj, *args, **kwargs):
    """ create clean JSON string. """
    from meds.utils.serialize import smooth
    return json.dumps(obj, default=smooth, *args, **kwargs)

## REGISTER

def reg(obj, cmnd, func):
    """ register a command, function combination. """
    from meds.utils.trace import get_plugname
    from meds.object import Object
    o = Object()
    o.cmnd = cmnd
    o.func = func
    o.plugname = get_plugname()
    cur = getattr(obj, o.cmnd, None)   
    if type(cur) in [types.FunctionType, types.MethodType, types.BuiltinFunctionType, types.BuiltinMethodType]: raise IsMethod(o.cmnd)
    if o.cmnd not in obj: obj[o.cmnd] = []
    todo = []
    for c in obj[o.cmnd]:
        if o.cmnd == c.cmnd and o.plugname == c.plugname: todo.append(c)
    for c in todo: obj[o.cmnd].remove(c)
    obj[o.cmnd].append(o)
    logging.info("register %s" % o.cmnd)

## HELPERS

def urled(obj):
    """ url of the Object's file. """
    from meds.cfg import cfg
    if "_path" in obj: fn = os.sep.join(obj._path.split(os.sep)[-2:])
    else: fn = ""
    return "http://%s:%s/%s" % (cfg.rest.hostname, cfg.kernel.port, fn)

def root(obj):
    """ return the working directory. """
    from meds.utils.file import check_permissions
    from meds.cfg import cfg
    if "workdir" in obj: path = obj.workdir
    else: path = cfg.kernel.workdir
    path = os.path.abspath(path)
    check_permissions(path)
    return path

def path(obj):
    """ return the path to the object. """
    from meds.utils.join import j
    p = root(obj)
    if "prefix" in obj: p = j(p, obj.prefix)
    return os.path.abspath(p) 

def selector(obj, keys):
    """ see if this objects has the desired attributes. """
    for key in keys:
        if key == "time": continue
        if key in obj: return True
        if not getattr(obj, key, None): return False
    return True

def wanted(obj, want):
    """ see if this objects has the desired attributes with their desired values. """
    go = True
    for key, value in want.items():
        if key == "format": continue
        if key not in obj: continue
        if value.startswith("-"): continue
        if value not in str(obj[key]): go = False ; break 
    return go

def notwanted(obj, notwant):
    """ see if this object doesn't have not desired key/values. """
    for key, value in notwant.items():
        if key == "format": continue
        if key not in obj: continue
        if value in obj[key]: return True
    return False

def days(obj):
    """ get the number of days relative to the ding's creation time. """
    from meds.utils.time import elapsed
    t1 = time.time()
    t2 = timed(obj)
    if t2:
        time_diff = float(t1 - t2)
        return elapsed(time_diff)

def dated(obj):
    """ retrieve the creation time of this object. """
    val = ""
    if "Date" in obj: val = obj.Date
    elif "date" in obj: val = obj.date
    elif "published" in obj: val = obj.published
    elif "added" in obj: val = obj.added
    elif "saved" in obj: val = obj.saved
    elif "timed" in obj: val = obj.timed
    return val

def timed(obj):
    """ retrieve the time stored in the path name of this object. """
    from meds.utils.time import short_date, to_time
    t = short_date(dated(obj))
    if t: t = to_time(t)
    if not t and "_path" in obj: t = fn_time(obj._path)
    if not t: t = fn_time(rtime())
    return t

def search(obj, name):
    """ search all attribute names containing substring txt. """
    l = []
    for key in names(obj):
        if name in key: l.append(key)
    return l

def match(obj, txt):
    """ match all object based on the last value of their dotted keynames. """
    try: return obj[txt]
    except KeyError: pass
    result = []
    for x in set(search(obj, txt)):
        if x.split(".")[-1] != txt: continue
        o = obj[x]
        if type(o) in [list, tuple]: result.extend(o) ; continue
        result.append(o)
    return result

def names(obj):
    """ provide keys suitable for iteration, skipping keys starting with a underscore. """
    try: keys = obj.keys()
    except AttributeError: return obj  
    for key in keys:
        k = str(key)
        if k.startswith("_"): continue
        yield k

def slice(obj, keys=[]):
    """ provide a sub directory based of a optional provided key list. """
    if not obj: return
    from meds.object import Object
    o = Object()
    if not keys: keys = names(obj)
    for key in keys:
        if key.startswith("_"): continue
        try: val = obj[key]
        except KeyError: continue
        try: val.keys() ; o[key] = slice(val)
        except: o[key] = val
    return o

