# time.py
#
#

""" time related functions. """

__copyright__ = "Copyright 2015, Bart Thate"

## IMPORT

import calendar
import datetime
import logging
import time
import os
import re

## DEFINEs

timere = re.compile('(\S+)\s+(\S+)\s+(\d+)\s+(\d+):(\d+):(\d+)\s+(\d+)')
bdmonths = ['Bo', 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
monthint = {
            'Jan': 1,
            'Feb': 2,
            'Mar': 3,
            'Apr': 4,
            'May': 5,
            'Jun': 6,
            'Jul': 7,
            'Aug': 8,
            'Sep': 9,
            'Oct': 10,
            'Nov': 11,
            'Dec': 12 
           }

## TIME

def fn_time(daystr):
    try:
        daystr = daystr.replace("_", ":")
        datestr = " ".join(daystr.split(os.sep)[-2:])
        datestr = datestr.split(".")[0]
        t = time.mktime(time.strptime(datestr, "%Y-%m-%d %H:%M:%S"))
        return t
    except: return 0

def rtime(): return str(datetime.datetime.now()).replace(" ", os.sep).replace(":", "_")
def hms(): return str(datetime.datetime.today()).split()[1].split(".")[0]
def day(): return str(datetime.datetime.today()).split()[0]
def to_day(daystr): return time.mktime(time.strptime(daystr, "%Y-%m-%d"))

def to_time(daystr):
    daystr = daystr.replace("_", ":")
    return time.mktime(time.strptime(daystr, "%Y-%m-%d %H:%M:%S"))

def short_date(*args, **kwargs):
    # Mon, 25 Oct 2010 18:05:33 -0700 (PDT)
    # ['13', 'Oct', '2012', '20:43:46', '+0300']
    date = args[0]
    if not date: return None
    date = date.replace("_", ":")
    res = date.split()
    ddd = ""
    try:
        if "+" in res[3]: raise ValueError
        if "-" in res[3]: raise ValueError
        int(res[3])
        ddd = "{:4}-{:#02}-{:#02} {:6}".format(res[3], monthint[res[2]], int(res[1]), res[4])
    except (IndexError, KeyError, ValueError):
        try:
            if "+" in res[4]: raise ValueError
            if "-" in res[4]: raise ValueError
            int(res[4])
            ddd = "{:4}-{:#02}-{:02} {:6}".format(res[4], monthint[res[1]], int(res[2]), res[3])
        except (IndexError, KeyError, ValueError):
            try: ddd = "{:4}-{:#02}-{:02} {:6}".format(res[2], monthint[res[1]], int(res[0]), res[3])
            except (IndexError, KeyError):
                try: ddd = "{:4}-{:#02}-{:02}".format(res[2], monthint[res[1]], int(res[0]))
                except (IndexError, KeyError): ddd = ""
    return ddd.replace(":", "_")

def elapsed(seconds,short=True):
    txt = ""
    nsec = int(float(seconds))
    year = 365*24*60*60
    week = 7*24*60*60
    day = 24*60*60
    hour = 60*60
    minute = 60
    years = int(nsec/year)
    nsec -= years*year
    weeks = int(nsec/week)
    nsec -= weeks*week
    days = int(nsec/day)
    nsec -= days*day
    hours = int(nsec/hour)
    nsec -= hours*hour
    minutes = int(nsec/minute)
    sec = nsec - minutes*minute
    if years: txt += "%sy" % years
    if weeks: days += weeks * 7
    if days: txt += "%sd" % days
    txt += " "
    if hours: txt += "%sh" % hours 
    if minutes: txt += "%sm" % minutes
    if sec and not hours: txt += "%ss" % sec
    if txt: return txt.strip()
    else: return "0s"

def get_day(daystr):
    """ convert string to time. """
    try:
        dmyre = re.search('(\d+)-(\d+)-(\d+)', daystr)
        (day, month, year) = dmyre.groups()
        day = int(day)
        month = int(month)
        year = int(year)
        if day <= calendar.monthrange(year, month)[1]:
            date = "%s %s %s" % (day, bdmonths[month], year)
            return time.mktime(time.strptime(date, "%d %b %Y"))
    except AttributeError: return 0
    except ValueError: return 0

def get_hour(daystr):
    try:
        hmsre = re.search('(\d+):(\d+):(\d+)', daystr)
        hours = 60 * 60 * (int(hmsre.group(1)))
        hoursmin = hours  + int(hmsre.group(2)) * 60
        hms = hoursmin + int(hmsre.group(3))
    except AttributeError: pass
    except ValueError: pass
    try:
        hmre = re.search('(\d+):(\d+)', daystr)
        hours = 60 * 60 * (int(hmre.group(1)))
        hms = hours + int(hmre.group(2)) * 60
    except AttributeError: return 0
    except ValueError: return 0
    return hms

## today function

def today():
    """ return time of 0:00 today. """
    today = day()
    ttime = time.strptime(today, "%Y-%m-%d")
    result = time.mktime(ttime)
    return result

